# Generated by roxygen2: do not edit by hand

S3method(plot,factor_select)
S3method(print,factor_select)
S3method(summary,factor_select)
export(select_factors)
export(simulate_factor_model)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(stats,rnorm)
importFrom(stats,sd)
