% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_factors.R
\name{print.factor_select}
\alias{print.factor_select}
\title{Print Method for factor_select Objects}
\usage{
\method{print}{factor_select}(x, ...)
}
\arguments{
\item{x}{A \code{factor_select} object.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{x}, called for its side effect of
printing a summary of the factor selection results to the console.
}
\description{
Print Method for factor_select Objects
}
